/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008,2011          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#ifndef _HFIDD_CLIENT_H_
#define _HFIDD_CLIENT_H_

static char sccsid_hfidd_client_h[] = "@(#) 1.49 src/hfi/dd/common/include/hfidd_client.h, ipdd.dd, ipdd_rchar, rchar1225a 12/07/10 21:01:34";

#define HFIDD_USER_ABI_VERSION  2

#define HFIDD_DD1               0x10
#define HFIDD_DD2               0x20
#define HFIDD_DD21              0x21


/*
 * For AIX, we will use the ioctl() DD entry point for these requests. Use
 *	HFIDD_REQ_* for the ioctl command.
 * For Linux, new ioctls are not allowed.  We will use write() calls to pass
 *	in an ioctl-looking request, with struct hfi_req_hdr giving the
 *	information we used to get from the ioctl() parameter list.  The
 *	write() call will copy out the request structure to the buffer pointed
 *	to by result, which is probably the original request.  The ioctl()
 *	command goes in req.
 */

struct hfi_64b {
	union {
		unsigned long long	allu;	/* APPLICATION Long long
						   Unsigned 64 bit address
						   container */
		void			*kptr;	/* KERNEL Pointer 64 bit
						   container */
	} use;
};

/* Request header: first structure in each of the HFI DD requests */
struct hfi_req_hdr {
	unsigned int	req;			/* HFIDD_REQ_* */
	unsigned int	req_len;		/* length of req, in bytes */
	unsigned int    abi_version;            /* ABI version */
	struct hfi_64b	result;			/* user eaddr for output
						   (Linux) */
};

#define HFIDD_REQ_HDR_SIZE			sizeof(struct hfi_req_hdr)

struct fifo_info {
	struct hfi_64b		eaddr;
	unsigned long long	size;		/* bytes */
};

struct hfi_rdma_submr {
	struct hfi_64b		eaddr;			/* starting addr for
							   this subMR's range */
	unsigned long long	size;			/* max for this subMR */
	unsigned long long	primary_page_size;	/* bytes */
	unsigned long long	secondary_page_size;	/* bytes */
	unsigned int		enable_local_write;
	unsigned int		enable_remote_write;
	unsigned int		enable_remote_read;
	unsigned int		enable_remote_atomic;
	unsigned int		lkey;
	unsigned long long	primary_bit_map_addr;
	unsigned long long	secondary_bit_map_addr;
};

/*
 * HFIDD_REQ_OPEN_WINDOW: Window Open
 * For reserved windows
 *	The job scheduler provides the application with window number and
 *	jobid, which need to be passed in/will be checked on the window open.
 *	Immediate send slots were also reserved by the job scheduler, and the
 *	DD returns the count. The application has the option of setting
 *      protection keys by task or by jobid. Protection key is passed in.
 *      The jobid must not have the top bit set.
 * For Dynamic windows
 *	The DD will select a free dynamic window and return its window number.
 *	The jobid will be assigned by the DD (upper bits set to select a
 *	reserved range of jobids, lower bits come from userid).  The
 *	protection key flag is always by task. The protection key is passed in.
 *      For non-kernel windows, the jobid is constructed with the with 
 *      most 2 significant bits set to 10, followed by the last six bits of 
 *      the user requested jobid, followed by the 24 least significant bits
 *      of the caller's UID.
 * For Kernel/IP windows
 *	The DD will select a free dynamic window and return its window number.
 *	The jobid is passed in.	The jobid is used for the protection key.
 *	Immediate send is not supported.
 *
 * Finished vectors:
 *	sfifo_finishvec and imm_finishvec must reside in an extra page (last
 *	page) in the sfifo memory range, provided by the user.  Similarly,
 *	rdma_finshvec must reside in an extra page (last page) in the
 *	rdma_sfifo memory range.
 *
 * RDMA/rCxt sharing:
 *	Within a jobid, scheduled windows can share RDMA mapping space and/or
 *	rCxt space.  Protection key flags should be "by jobid".
 *	The first window to open should set RDMA_shared_with and/or
 *	rCxt_shared_with to MAX_WIN_PER_HFI. The 2nd-Nth window to open specify
 *	an already opened window (probably the first one).
 *	By setting the protection key to "by jobid" and having
 *	non-zero values in RDMA_shared_with and/or rCxt_shared_with tells the DD
 *	that you want to share those resources with other windows in your jobid.
 */

#define HFIDD_IP_WIN		1	/* IP windows get broadcasts forwarded
						to them... */
#define HFIDD_KERNEL_WIN	2	/* ... other kernel windows do not */
#define HFIDD_RESERVE_WIN	3	/* Must be reserved by job scheduler */
#define HFIDD_DYNAMIC_WIN	4	/* First come, first served. Window# is
						returned */

#define HFIDD_PROTECTION_KEY_BYTASK	1
#define HFIDD_PROTECTION_KEY_BYJOBID	2

#define HFIDD_DST_BCST_WIN		0
#define HFIDD_DST_BCST_ISR		0x3FFF

#define DYNAMIC_JOBID_MASK		0x80000000

struct hfi_client_info {
	struct hfi_req_hdr	hdr;

	unsigned int		window;
	unsigned int		win_type;		/* HFIDD_*_WIN */
	unsigned int		job_id;
	unsigned int		protection_key_flag;
	unsigned int		protection_key;
	unsigned int		immediate_send_slots;
	unsigned int		local_isrid;		/* Output */
	struct fifo_info	sfifo;
	struct fifo_info	rfifo;
	struct fifo_info	rdma_sfifo;		/* size = 0 if not
							   required */
	struct fifo_info	ip2k_fifo;		/* size = 0 if not
							   required */
	struct fifo_info	desc_buffers;		/* size = 0 if not
							   required */
	struct hfi_64b		sfifo_finish_vec;
	struct hfi_64b		rdma_finish_vec;
	struct hfi_64b		imm_finish_vec;
	unsigned int		imm_send_update_freq;
	unsigned int		fullrdma_msg_breakup_count;
	unsigned int		rdmapending_read_req_thresh;
	uint			desc_buffers_lkey;	/* Output	   */
	uint			sfifo_lkey;		/* Output	   */
	struct hfi_64b		mmio_regs;		/* Output	   */
#ifdef PANEFUL
	struct hfi_64b		mmio_fifo;		/* Output	   */
#endif

	unsigned int		rdma_shared_with;	/* window# to share
							   RDMA space with */
	unsigned int		rcxt_shared_with;	/* window# to share
							   rcxt space with */
	unsigned int		rcxt_count;
	unsigned int		initial_epoch;
	unsigned int		sendspecial_fifo_size;	/* size = 0 if not
							   required */
	unsigned int		rdmapending_fifo_size;	/* size = 0 if not
							   required */
	unsigned int		rdma_map_max_size;
	struct hfi_rdma_submr	submr0;			/* SubMR 0 is allocated
							   with the superMR at
							   window open.	 size =
							   0 if not required */
};

/*
 * HFIDD_REQ_CLOSE_WINDOW: close window
 * HFIDD_REQ_SUSPEND_WINDOW: suspend window
 * HFIDD_REQ_RESUME_WINDOW: resume window
 */
struct hfi_window_info {
	struct hfi_req_hdr	hdr;
	unsigned int		window;
};

/* HFIDD_REQ_CHANGE_PARM: "chdev" parms */
/* Add defines for ODM/sysconfig stuff here */
#define HFIDD_PARM_DD_TRACE_LEVEL		1
#define HFIDD_PARM_IP_TRACE_LEVEL		2
#define HFIDD_PARM_MAX_RDMA_SIZE		3
#define HFIDD_PARM_MAX_RCXT_SIZE		4
#define HFIDD_PARM_MAX_SSF_SIZE			5
#define HFIDD_PARM_NUM_DYNAMIC_WIN		6	/* max dynamic window */
/* etc... */
struct hfi_chdev {
	struct hfi_req_hdr	hdr;
	unsigned int		which_parm;		/* HFIDD_PARM_* */
	unsigned long long	new_value;		/* Are they really all
							   ULLs/no strings? */
};

/* HFIDD_REQ_GET_PERF_STATS: get perf stats */
struct hfi_perf_stats {
	struct hfi_req_hdr	hdr;

	/* HFI */
	unsigned long long	aggregate_packets_sent;
	unsigned long long	aggregate_packets_dropped_sending;
	unsigned long long	aggregate_packets_received;
	unsigned long long	aggregate_packets_dropped_receiving;
	unsigned long long	aggregate_immediate_send_packet_count;
	unsigned long long	aggregate_send_recv_packet_count;
	unsigned long long	aggregate_fullrdma_sent_count;
	unsigned long long	aggregate_halfrdma_sent_count;
	unsigned long long	aggregate_smallrdma_sent_count;
	unsigned long long	aggregate_ip_packet_sent_count;
	unsigned long long	aggregate_cau_packet_sent_count;
	unsigned long long	aggregate_gups_packet_sent_count;
	unsigned long long 	aggregate_notifications_packet_sent_count;
	unsigned long long	addr_xlat_wait_count;
};

/*
 * HFIDD_REQ_MAP_GLOBAL_CTR/HFIDD_REQ_UNMAP_GLOBAL_CTR: global counter map/unmap
 */
struct hfi_map_global_ctr {
	struct hfi_req_hdr	hdr;
	struct hfi_64b		eaddr;		/* output */
};

/* RDMA stuff */
struct hfi_rdma_reg_entry {
	struct hfi_64b		eaddr;		/* start address */
	unsigned long long	length;		/* bytes */
	unsigned int		submr;
	unsigned long long	page_size;
	struct hfi_64b		err_page;	/* which one failed to map */
};

/* HFIDD_REQ_RDMA_REG/HFIDD_REQ_RDMA_UNREG: RDMA reg/unreg */
#define HFIDD_MAX_RDMA_REG_ENTRIES (1024 / sizeof(struct hfi_rdma_reg_entry))
struct hfi_rdma_reg {
	struct hfi_req_hdr		hdr;
	unsigned int			window;
	unsigned int			count;	/* number of ranges to map */
	struct hfi_rdma_reg_entry	range[HFIDD_MAX_RDMA_REG_ENTRIES];
};

/*
 * HFIDD_REQ_RDMA_ALLOC_SUBMR/HFIDD_REQ_RDMA_FREE_SUBMR: RDMA submr alloc/free
 */
struct hfi_rdma_submr_req {
	struct hfi_req_hdr	hdr;
	unsigned int		window;
	unsigned int		which;		/* submr# */
	struct hfi_rdma_submr	submr;
};

/*
 * HFIDD_REQ_RDMA_RCXT_READ: read rCxt
 * HFIDD_REQ_RDMA_RCXT_UPDATE_EPOCH: rCxt epoch update)
 */
#define HFIDD_RCXT_SIZE		128
struct hfi_rcxt_op {
	struct hfi_req_hdr	hdr;
	unsigned int		window;
	unsigned int		epoch;
	unsigned long long	offset;
	char			rcxt[HFIDD_RCXT_SIZE];
};

/*
 * Event Notification stuff
 * todo: post2clock_readers
 * todo: register_swclock_et
 */
enum hfi_event_type {
	HFIDD_SEND		= 0,
	HFIDD_RECV		= 1,
	HFIDD_WIN_ERROR		= 2,
	HFIDD_HFI_ERROR		= 3,
	HFIDD_TERMINATE		= 4,
	HFIDD_RELEASE_WINDOW	= 5,
	HFIDD_CAU_ERROR		= 6,
	HFIDD_ICS_ERROR		= 7,
	HFIDD_HFI_READY_REG	= 8,
	HFIDD_ROUTE_CHANGE	= 9,
	HFIDD_IP_TRC_LVL	= 10,  /* IP Window only */
	HFIDD_POOL_SIZE		= 11,  /* IP Window only */
	HFIDD_CAU_REC_ERROR	= 12,
	HFIDD_IP_SEND_LIMIT	= 13,  /* IP Window only */
	HFIDD_IP_RECV_LIMIT	= 14,  /* IP Window only */
	HFIDD_NUM_EVENT_TYPES	= 15
};

#define HFIDD_SEND_EVENT			0x00000100
#define HFIDD_RECV_EVENT			0x00000200
#define HFIDD_WIN_ERROR_EVENT			0x00000400
#define HFIDD_HFI_ERROR_EVENT			0x00000800
#define HFIDD_TERMINATE_EVENT			0x00001000
#define HFIDD_RELEASE_WINDOW_EVENT		0x00002000
#define HFIDD_CAU_ERROR_EVENT			0x00004000
#define HFIDD_ICS_ERROR_EVENT			0x00008000
#define HFIDD_HFI_READY_REG_EVENT		0x00010000
#define HFIDD_ROUTE_CHANGE_EVENT		0x00020000
#define HFIDD_IP_TRC_LVL_EVENT			0x00040000
#define HFIDD_POOL_SIZE_EVENT			0x00080000
#define HFIDD_CAU_REC_ERROR_EVENT		0x00100000
#define HFIDD_LOWEST_EVENT			HFIDD_SEND_EVENT
#define HFIDD_ALL_EVENTS			0x000FFF00

enum hfi_event_hndlr_type {
	WAIT_FOR_EVENTS		= 1,	/* Wait for events */
	SIGNAL_EVENTS		= 2,	/* Event notification by signal */
	FUNCTIONS_FOR_EVENTS	= 3	/* Callback functions */
};

typedef int (*hfi_event_func_ptr)(void *parm, unsigned int win,
		unsigned int *ext);

enum hf_pool_mreq {		/* Value of ext for hfi_event_func_ptr on
				   calls to ip to set pool size. */
	CHG_SPOOL = 0x91,
	CHG_RPOOL = 0x92
};

struct hfi_callback_func {	   /* Callback funcs for kernel windows */
	enum hfi_event_type	index;		/* index of callback type */
	unsigned int		pad;
	struct hfi_64b		function_p;	/* function ptr */
	struct hfi_64b		parameter;	/* parameter to pass in */
};

/*
 * HFIDD_REQ_EVENT_REGISTER/HFIDD_REQ_EVENT_UNREGISTER: event
 * registration/unregistration
 */
#define HFI_MAX_BUF_EVENTS	8	/* # event buffers for signal version */
struct hfi_event_buffer	 {		/* For reporting events with signals */
	unsigned int		tag;
	enum hfi_event_type	current_event;
};

struct hfi_reg_events {
	struct hfi_req_hdr		hdr;
	unsigned int			window;
	enum hfi_event_hndlr_type	type;
	union {
		unsigned int		events;
		struct hfi_callback_func func;
		struct hfi_64b		 eb; 	/* Pointer to event buffer in
						 user space (signal only) */
	} info;
};

/* HFIDD_REQ_EVENT_WAIT: wait on event */
struct hfi_wait_events {
	struct hfi_req_hdr	hdr;
	unsigned int		window;		/* Window for events */
	unsigned int		events;		/* events to wait for */
	unsigned int		out_events;	/* events received */
};

/* Job Scheduler stuff */
/*
 * HFIDD_REQ_RESERVE_WINDOW/HFIDD_REQ_UNRESERVE_WINDOW:
 */
struct hfi_reserve_window {
	struct hfi_req_hdr	hdr;
	unsigned int		window;
	uid_t			uid;
	unsigned int		job_id;
	unsigned int		immediate_send_slots;
};

struct hfi_query_window {
	unsigned int		window;
	unsigned int		job_id;
	unsigned char		in_use;
	unsigned char		schedulable;
	unsigned char		blah[2];		/* Padding */
};

struct hfi_query_cau_slot {
	unsigned int		job_id;
	unsigned int		index;
	unsigned char		in_use;
	unsigned char		blah[3];		/* Padding */
};

/* HFIDD_REQ_QUERY_RESOURCE: query resources */
struct hfi_query_resource {
	struct hfi_req_hdr	hdr;
	unsigned int		immediate_send_avail;
	unsigned int		immediate_send_max;
	unsigned int		dynamic_window_avail;
	unsigned int		dynamic_window_max;
	unsigned int		window_count;	/* in: array len, out: actual
						   used */
	unsigned int		cau_slot_count;	/* in: array len, out: actual
						   used */
	struct hfi_64b		windows;	/* in: ptr to array of struct
						   hfi_query_window */
	struct hfi_64b		cau_slots;	/* in: ptr to array of struct
						   hfi_query_cau_slot */
	unsigned int		local_isr_id;	/* Output */
};

struct hfi_adpt_resource {

	struct hfi_req_hdr	hdr;
	unsigned int		ip_trc_lvl;
	unsigned int		spool_size;
	unsigned int		rpool_size;
	unsigned int		send_limit;
	unsigned int		recv_limit;
};

/* HFIDD_REQ_SIGNAL_WINDOW: signal window/jobid */
struct hfi_signal_window {
	struct hfi_req_hdr	hdr;
	unsigned int		which;		/* sig window: window#
						   sig jobid: jobid */
	unsigned int		signal;
};


/* HFIDD_REQ_SOFT_DOWN: software hfi/window down */
struct hfi_softdown_req {
	struct hfi_req_hdr	hdr;
	unsigned int		hfi_id;
	unsigned int		win_id;
	char			err_str[80];
	unsigned char		subsys_id;
};

/* HFIDD_REQ_CAU_RESERVE/HFIDD_REQ_CAU_UNRESERVE: reserve/unreserve CAU slot */
struct hfi_reserve_cau {
	struct hfi_req_hdr	hdr;
	unsigned int		index;
	unsigned int		job_id;
};

/* CAU stuff */
struct hfi_cau_entry {
	unsigned int		state;
	unsigned int		input_recv;
	unsigned int		sent_ack;
	unsigned int		recv_ack;
	unsigned int		output_sent;
	unsigned int		sequence_num;
	unsigned int		pkt_type;
	unsigned int		pkt_op_attr;
	unsigned int		pkt_func;
	unsigned int		payload_present;
	unsigned long long	cookie_left;
	unsigned long long	cookie_right;
	unsigned long long	payload_field[8];
	unsigned int		derr_array;
};

struct hfi_cau_neighbor {
	unsigned int		sub_id;
	unsigned int		isr;
	unsigned int		is_cau;
	unsigned int		window;
	unsigned int		route_control;
};

/* CAU */
#define MAX_NEIGHBORS_PER_CAU		9
#define MAX_ENTRIES_PER_CAU_INDEX	2
struct hfi_cau_tree {
	unsigned int		valid;
	unsigned long long	tree_id;
	unsigned int		neighbor_count;
	unsigned int		sequence_num_a;
	unsigned int		sequence_num_b;
	struct hfi_cau_neighbor	neighbors[MAX_NEIGHBORS_PER_CAU];
	struct hfi_cau_entry	entry[MAX_ENTRIES_PER_CAU_INDEX];
	unsigned long long	cycles_waiting_for_credit;
	unsigned long long	packet_count;
	unsigned long long	packet_dropped_count;
};

/* HFIDD_REQ_CAU_READ/HFIDD_REQ_CAU_WRITE: read/write CAU slot */
struct hfi_cau_rw {
	struct hfi_req_hdr	hdr;
	unsigned int		index;
	unsigned int		job_id;
	struct hfi_cau_tree	tree;
};

/* HFIDD_REQ_DBG_TRC */
/* type field */
#define	HFI_DUMP_DBG_HNDL	0x00A0
#define	HFI_DUMP_DBG_TRC	0x00A1
#define	HFI_DUMP_DBG_INFO	0x00A2

struct hfi_dbg_trc {
	struct hfi_req_hdr	hdr;
	unsigned char		name[16];	/* hf0, hf1,... */
	unsigned int		type;		/* trc, info,.. */
	unsigned int		len; /* byte size of output trace buffer */
	struct hfi_64b		buf;
};

typedef int (*hfi_dbg_func_ptr)(void *parm, struct hfi_dbg_trc *dbg,
		unsigned short ext);

/* HFIDD_REQ_DBG_REG, HFIDD_REQ_DBG_UNREG */
struct hfi_reg_dbg_callback {
	struct hfi_req_hdr		hdr;
	unsigned char			name[16];	/* hf0, hf1,... */
	unsigned int			window;		/* window index */
	unsigned int			rsvd;
	hfi_dbg_func_ptr		function_p;	/* function ptr */
	void				*parameter;	/* paramter to pass
							   in */
};

/*
 * HFIDD_REQ_ATTACH: get direct addressability to registers from user space
 * (pre-GA only)
 */
#define HFIDD_ATTACH_HFI0		0
#define HFIDD_ATTACH_HFI1		1
#define HFIDD_ATTACH_NMMU_MRWPT		2
#define HFIDD_ATTACH_NMMU_GBA		3
#define HFIDD_ATTACH_CAU		4
struct hfi_attach {
	struct hfi_req_hdr	hdr;
	unsigned int		which;	/* HFI_ATTACH_* */
	struct hfi_64b		eaddr;	/* out */
};

/*
 * HFIDD_REQ_ROUTE_INFO
 */
struct hfi_route_req {
	struct hfi_req_hdr	hdr;
	unsigned int		route_per_dest;
	unsigned int		max_isr;
	struct hfi_64b		flag;	/* out, pointer to flag indicating
					   which of the 2 copies of the route
					   table is valid */
	struct hfi_64b		addr[2];/* out, pointers to the two route
					   tables (one is valid, other is
					   old or updating) */
};

/*
 * HFIDD_REQ_MCAST_ID_INFO
 *  mc_group_id_mask :  0x01=group_id(0), 0x02=group_id(1), 0x3=group_id(0 & 1)
 */
struct hfi_mcast_id_req {
	struct hfi_req_hdr	hdr;
	unsigned int		mc_group_id_mask;
};

/* 
 * HFIDD_REQ_QUERY_DD_INFO
 */
struct hfi_query_dd_info {
        struct hfi_req_hdr      hdr;
        unsigned long long      fw_ec_level;    /* Hardware Version */
        unsigned int            abi_version;    /* ABI Version  */
};


#define HFI_DYN_WINS_DEFAULT	32
/* Change attrible input */
struct hfi_chg_attr {
	struct hfi_req_hdr		hdr;
	unsigned int			val;
	unsigned int			ret;
};

typedef unsigned long long hfi_mlimits_size_t; /* Attribute value */
#define HFI_SPOOLSIZE_MIN	0x2000000
#define HFI_SPOOLSIZE_MAX	0x20000000
#define HFI_RPOOLSIZE_MIN	0x2000000
#define HFI_RPOOLSIZE_MAX	0x20000000
#define HFI_SEND_LIMIT_MIN	4
#define HFI_SEND_LIMIT_MAX	256
#define HFI_RECV_LIMIT_MIN	4
#define HFI_RECV_LIMIT_MAX	512

#define HFI_SUBMR_MAX           6

#ifdef MULTIPLE_TORRENTS
#define MAX_TORRENTS		8
#else
#define MAX_TORRENTS            1
#endif
#define MAX_HFI_PER_TORRENT     2
#define MAX_HFIS                (MAX_TORRENTS * MAX_HFI_PER_TORRENT)
#define MAX_WIN_PER_HFI         256
#define MAX_SLOTS_PER_CAU       64

#define PAGE_SIZE_4K            0x1000
#define PAGE_SIZE_64K           0x10000
#define PAGE_SIZE_1M            0x100000
#define PAGE_SIZE_16M           0x1000000
#define PAGE_SIZE_256M          0x10000000
#define PAGE_SIZE_4G            0x100000000
#define PAGE_SIZE_16G           0x400000000

#define HFI_MAX_ULONG		(0xFFFFFFFFFFFFFFFFUL)
#endif /* _HFIDD_CLIENT_H_ */
